//--------------------------------------------------------------------------------------------------------------------------------------------------------
//Stepper_Motor_Current_0v3.pov  
//Skrypt wykrelajcy wykresy narastania prdu w uzwojeniu silnika krokowego
//w zalenoci od jego parametrw i napicia przyoonego do jego uzwojenia.
//Prdko obrotowa [Rpm] podawana jest dla pracy peno-krokowej. 
//Autor: (2013) cyprian Kamil Kowalski c4v2@o2.pl  
//Licencja: GNU.
//--------------------------------------------------------------------------------------------------------------------------------------------------------
//Wymagane parametry wiersza polece POV-Ray:
//+LC:\WINDOWS\Fonts\ -j +q11 +FN                                         

//--------------------------------------------------------------------------------------------------------------------------------------------------------
//Parametry silnika: 
//np. 57BYGH802     //57BYG081 

//[ Lm  ] Indukcyjno uzwojenia [H]:
#declare Lm=0.0071; //0.00845; 

//[ Rm  ] Rezystancja uzwojenia [Ohm]:      
#declare Rm=3.2;    //5;

//[ Im  ] Prd roboczy uzwojenia (fazy) [A]:
#declare Im=1.5;    //1;
  
/*
//Gdy nie podano wartoci prdu roboczego uzwojenia [ Im ], lecz podano 
//[ Um  ] Napicie robocze uzwojenia [V]:
#declare Um=4.8;
//[ Im  ] warto prdu roboczego naley wyliczy:          
#declare Im=Um/Rm; 
*/
  
//[ Spr ] Ilo krokw na jeden obrt:         
#declare Spr=200; 
       
//[ Vcc ] Zakadane napicie zasilania silnika ("widziane" przez uzwojenie) [V]: 
//czyli warto napicia zasilania sterownika pomniejszona o spadek  
//napicia na rezystorze pomiarowym oraz w przypadku mostka H mocy: 
//opartego na tranzystorach MOSFET pomniejszone o spadek napicia na Ron,
//opartego na tranzystorach bipolarnych pomniejszone napicie nasycenia Vsat.
//W obu ww. przypadkach rnica dotyczy jednego tranzystora grnego i jednego dolnego w mostku.   
#declare Vcc=24;         

//--------------------------------------------------------------------------------------------------------------------------------------------------------
#declare ShowAll=off;  

//--------------------------------------------------------------------------------------------------------------------------------------------------------
#include "colors.inc"
#include "shapes.inc"
#include "strings.inc"  

//--------------------------------------------------------------------------------------------------------------------------------------------------------
background {Black} 
camera {orthographic location <5.5,4.5,-10> look_at <5.5,4.5,0>}  
light_source {<1000,1000,1000> White} 
light_source {<1000,1000,-1000> White} 
light_source {<1000,-1000,-1000> White} 

//--------------------------------------------------------------------------------------------------------------------------------------------------------
#macro Axe(m)
union {
  cylinder{-200*x,200*x,0.02 pigment{White}}   
  cylinder{-0.1*y,200*y,0.02 pigment{White}} 
  #local i=1; 
  #local ms=m/1000;
  #local l=ms;
  #while (i<20)
     cylinder{-0.1*y,0.1*y,0.02 pigment{White} translate<l,0,0>}
     #if (l<(Tau*X*3))  
       cylinder{0*y,(Im*Y)*y,0.015 pigment{Gray10} translate<l,0,0.15>} 
     #end
     text {ttf "COURBD.TTF" concat("", str(i,1,0)), 1, 0 scale<0.4,0.4,0.4> translate<l-0.05,-0.4,-0.05> pigment {White}}    
     #local i=i+1;
     #local l=l+ms;
  #end
  text {ttf "COURBD.TTF" "t[ms]", 1, 0 scale<0.4,0.4,0.4> translate<-1.1,-0.4,-0.05> pigment {White}} 
  text {ttf "COURBD.TTF" "I[A]", 1, 0 scale<0.4,0.4,0.4> translate<-0.9,(Im*Y)+0.5,-0.05> pigment {White}}  
  #local i=0;
  #local j=0; 
  #while (i<(Y*Im)) 
     #if ((mod(j,5)=0)&(j>0))
       cylinder{-0.2*x,0.2*x,0.02 translate<0,i,0> pigment{White}} 
       cylinder{0*x,(Tau*X*3)*x,0.015 translate<0,i,0.1> pigment{Gray10}}
       text {ttf "COURBD.TTF" concat("", str(j/10,1,1)), 1, 0 scale<0.4,0.4,0.4> translate<-1.05,(i)-0.1,-0.05> pigment {White}}   
     #end  
     #if (mod(j,10)=0)
       cylinder{-0.3*x,0.3*x,0.02 translate<0,i,0> pigment{White}}   
     #end    
     cylinder{-0.1*x,0.1*x,0.02 translate<0,i,0> pigment{White}}     
     #local j=j+1;
     #local i=i+(Y/10);
  #end                
}
#end

//--------------------------------------------------------------------------------------------------------------------------------------------------------  
#macro Ph_Current(I,U,R,L,Color,Width,i) 
union{ 
  #local e=2.718281828;
  #local T=0;
  #local Ip=0;  
  #while (T<5*Tau)                      
    #local Ip=((U/R)*(1-(pow(e,-(T/Tau)))));  
    #if ((T<3*Tau)&(Ip<I))
       sphere{0*x,Width translate<T*X,Ip*Y,0> pigment{Color}} 
       #declare t4Umax=T;
    #else
       sphere{0*x,0.02 translate<T*X,Ip*Y,0> pigment{Gray10}}        
    #end        
    #local T=T+i;   
  #end               
}
#end   

//--------------------------------------------------------------------------------------------------------------------------------------------------------
#local Tau=Lm/Rm;  
#local X=7.75/(Tau*3);   
#if (ShowAll=on)
  #local Y=8.5/((Vcc/Rm)*(1-(pow(e,-((4*Tau)/Tau)))));  
#else
  #local Y=8.5/Im; 
#end      
union{ 
  object {Ph_Current(Im,Rm*Im,Rm,Lm,Green,0.02,0.000002)}
  #local U=ceil(Rm*Im)+1; 
  #while (U<Vcc)
    object {Ph_Current(Im,U,Rm,Lm,Green-Gray90,0.02,0.000002) translate<0,0,0.1>} 
    #local U=U+1;
  #end  
  #if (Vcc>Rm*Im)
    object {Ph_Current(Im,Vcc,Rm,Lm,Blue,0.03,0.0000001)} 
  #end     
  #local Text=array[14];   
  #local Text[0]=concat("Im=", str(Im,1,2)," A");      
  #local Text[1]=concat("Lm=", str(Lm*1000,1,2)," mH"); 
  #local Text[2]=concat("Rm=", str(Rm,1,2)," Ohm");      
  #local Text[3]=concat("Um=", str(Rm*Im,1,2)," V");   
  #local Text[4]=concat("Spr=", str(Spr,1,0),"");
  #local Text[5]=concat("2*t=", str(Tau*6000,1,1)," ms");             
  #local Text[6]=concat("f=", str(1/(Tau*6),1,1)," Hz");              
  #local Text[7]=concat(str(((1/(Tau*6))*60)/Spr,1,1)," rpm");  
  #local Text[8]=concat("Vcc=", str(Vcc,1,2)," V");                           
  #local Text[9]=concat("t=", str(t4Umax*1000,1,2)," ms");                     
  #local Text[10]=concat("f=", str(1/(t4Umax*2),1,1)," Hz");             
  #local Text[11]=concat(str(((1/(t4Umax*2))*60)/Spr,1,1)," rpm"); 
  #local Text[12]=concat("Tau=", str(Tau*1000,1,2));  
  #local Text[13]=concat("3*Tau=", str(Tau*3000,1,2));     
  #local i=0;
  #while (i<=11) 
    #if (i>7) 
      #if (Vcc>Rm*Im) 
        cylinder{-0.1*y,((Im*Y)+0.2)*y,0.015 pigment{Blue-Gray50} translate<t4Umax*X,0,-0.01>}       
        text {ttf "COURBD.TTF" Text[i], 1, 0 scale 0.6 translate<(Tau*X*3)+0.2,8-(i*0.6),-0.1> pigment {Blue}}   
      #else
        text {ttf "COURBD.TTF" "Err Ur>=Vcc", 1, 0 scale 0.6 translate<(Tau*X*3)+0.2,8-(i*0.6),-0.1> pigment {Red}} 
        #local i=11;  
      #end   
    #else
      text {ttf "COURBD.TTF" Text[i], 1, 0 scale 0.6 translate<(Tau*X*3)+0.2,8-(i*0.6),-0.1> pigment {Green}}        
    #end
    #local i=i+1;
  #end 
  cylinder{0*x,((Tau*X*3)+0.1)*x,0.015 pigment{Red} translate<0,Im*Y,0>}
  cylinder{-0.1*y,((Im*Y)+0.2)*y,0.015 pigment{Cyan} translate<Tau*X,0,0>}
  cylinder{-0.1*y,((Im*Y)+0.1)*y,0.015 pigment{Gray50} translate<Tau*X*3,0,0>}   
  text {ttf "COURBD.TTF" Text[12], 1, 0 scale 0.6 translate<(Tau*X)-0.15,Im*Y+0.3,-0.1> pigment {Cyan}} 
  text {ttf "COURBD.TTF" Text[13], 1, 0 scale 0.6 translate<(Tau*X*3)+0.2,0.2,-0.1> pigment {Gray50}}  
  object {Axe(X)} 
  no_reflection
  no_shadow
} 

//Ende----------------------------------------------------------------------------------------------------------------------------------------------------
